package com.winhc.repal.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.winhc.common.model.base.VOPage;
import com.winhc.repal.constant.Constant;
import com.winhc.repal.entity.*;
import com.winhc.repal.enums.DeletedStatusEnum;
import com.winhc.repal.enums.RoleTypeEnum;
import com.winhc.repal.model.dto.InitGroupDTO;
import com.winhc.repal.model.dto.MemberRoleDTO;
import com.winhc.repal.model.dto.PageDTO;
import com.winhc.repal.model.vo.GroupMemberVo;
import com.winhc.repal.repository.RepalGroupMapper;
import com.winhc.repal.service.*;
import com.winhc.repal.util.UserContextUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author Generator
 * @description RepalGroup 接口实现类
 * @date 2022-04-13
 */
@Service
public class RepalGroupServiceImpl extends ServiceImpl<RepalGroupMapper, RepalGroup> implements RepalGroupService {

    private static final Logger LOGGER = LoggerFactory.getLogger(RepalGroupServiceImpl.class);

    @Autowired
    private RepalGroupMemberService repalGroupMemberService;

    @Autowired
    private RepalBookService repalBookService;

    @Autowired
    private RepalRolePermissionService repalRolePermissionService;

    @Autowired
    private RepalGroupRoleService repalGroupRoleService;

    @Autowired
    private UserExtService userExtService;

    @Autowired
    private RepalVipService repalVipService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean initGroup(InitGroupDTO dto) {
        UserExt userExt = userExtService.getById(dto.getUserId());
        // member判断
        if (repalGroupMemberService.count(Wrappers.lambdaQuery(RepalGroupMember.class)
                .eq(RepalGroupMember::getUserId, dto.getUserId())
                .eq(RepalGroupMember::getDeleted, DeletedStatusEnum.NORMAL.getCode())) > 0) {
            LOGGER.info("已经属于组织成员了, userId:[{}]", dto.getUserId());
            return true;
        }

        RepalGroup repalGroup = this.getOne(Wrappers.lambdaQuery(RepalGroup.class)
                .eq(RepalGroup::getUserId, dto.getUserId())
                .eq(RepalGroup::getDeleted, DeletedStatusEnum.NORMAL.getCode()));
        if (Objects.isNull(repalGroup)) {
            repalGroup = new RepalGroup();
            repalGroup.setGroupName(dto.getGroupName());
            repalGroup.setUserId(dto.getUserId());
            repalGroup.setGroupExt(dto.getGroupExt());
            this.save(repalGroup);
            // 初始化4个角色
            for (int i = 0; i < Constant.INIT_ROLE_NAME_LIST.size(); i++) {
                MemberRoleDTO memberRoleDTO = new MemberRoleDTO();
                memberRoleDTO.setRoleName(Constant.INIT_ROLE_NAME_LIST.get(i));
                memberRoleDTO.setPermissionCodeList(Constant.INIT_ROLE_LIST.get(i + 1));
                repalGroupRoleService.addRole(memberRoleDTO);
            }
            // 组织初始化时初始化一个账本
            RepalBook repalBook = new RepalBook();
            repalBook.setRepalGroupId(repalGroup.getId());
            repalBook.setRepalBookName(dto.getGroupName());
            repalBook.setUserId(dto.getUserId());
            repalBookService.save(repalBook);
        }
        // 添加到管理员
        repalGroupMemberService.addGroupMember(dto.getUserId(), userExt.getUserName(), repalGroup.getId(), 1L, RoleTypeEnum.SYSTEM.getCode());
        return true;
    }

    @Override
    public VOPage<GroupMemberVo> getGroupMemberList(PageDTO dto) {
        VOPage<GroupMemberVo> voPage = new VOPage<>();
        voPage.setPageSize(dto.getPageSize());
        voPage.setPageNum(dto.getPageNum());
        Page<RepalGroupMember> page = new Page<>(dto.getPageNum(), dto.getPageSize());
        page = repalGroupMemberService.page(page, Wrappers.lambdaQuery(RepalGroupMember.class)
                .eq(RepalGroupMember::getRepalGroupId, repalGroupMemberService.getUserGroupId(UserContextUtil.getUser().getUserId()))
                .eq(RepalGroupMember::getDeleted, DeletedStatusEnum.NORMAL.getCode()));
        voPage.setTotalPage(Math.toIntExact(page.getPages()));
        voPage.setTotalNum(page.getTotal());
        if (CollUtil.isEmpty(page.getRecords())) {
            voPage.setDataList(new ArrayList<>());
            return voPage;
        }
        voPage.setDataList(page.getRecords().stream().map(t -> new GroupMemberVo(t.getUserId(), t.getMemberUserName())).collect(Collectors.toList()));
        return voPage;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean vipCallback(Long repalGroupId) {
        RepalVip repalVip = new RepalVip();
        repalVip.setRepalGroupId(repalGroupId);
        //repalVip.setRepalVipSign();
        repalVip.setRepalVipStartTime(LocalDateTime.now());
        //repalVip.setRepalVipEndTime();
        //repalVip.setCurrentOrder();
        // remainCount
//        repalVip.setSmsRemainCount();
//        repalVip.setLetterRemainCount();
//        repalVip.setConsultRemainCount();
//        repalVip.setLawsuitRemainCount();
        // total
//        repalVip.setSmsTotalCount();
//        repalVip.setLetterTotalCount();
//        repalVip.setConsultTotalCount();
//        repalVip.setLawsuitTotalCount();
        repalVipService.save(repalVip);
        return true;
    }
}
